@@ -1,7 +1,7 @@
 import argparse
 import telnetlib
 
-from .aiomonitor import MONITOR_HOST, MONITOR_PORT
+from .monitor import MONITOR_HOST, MONITOR_PORT
 
 
 def monitor_client(host, port):
@@ -16,14 +16,14 @@ def monitor_client(host, port):
 
 
 def main():
-    parser = argparse.ArgumentParser("usage: python -m aiomonitor [options]")
-    parser.add_argument("-H", "--host", dest="monitor_host",
+    parser = argparse.ArgumentParser('usage: python -m aiomonitor [options]')
+    parser.add_argument('-H', '--host', dest='monitor_host',
                         default=MONITOR_HOST, type=str,
-                        help="monitor host ip")
+                        help='monitor host ip')
 
-    parser.add_argument("-p", "--port", dest="monitor_port",
+    parser.add_argument('-p', '--port', dest='monitor_port',
                         default=MONITOR_PORT, type=int,
-                        help="monitor port number")
+                        help='monitor port number')
     args = parser.parse_args()
     monitor_client(args.monitor_host, args.monitor_port)
 
