@@ -3,13 +3,18 @@ import logging
 import os
 import signal
 import socket
+import sys
 import threading
+import traceback
 from textwrap import wrap
+from typing import IO, Dict, Any, Optional  # noqa
+from concurrent.futures import Future  # noqa
 
 from terminaltables import AsciiTable
 
 from .utils import (_format_stack, cancel_task, task_by_id, console_proxy,
                     init_console_server)
+from .mypy_types import Loop, OptLocals
 
 
 __all__ = ('Monitor', 'start_monitor')
@@ -21,69 +26,84 @@ MONITOR_PORT = 50101
 CONSOLE_PORT = 50102
 
 
-def start_monitor(loop, *, host=MONITOR_HOST, port=MONITOR_PORT,
-                  console_port=CONSOLE_PORT, console_enabled=True):
+def start_monitor(loop: Loop, *,
+                  host: str=MONITOR_HOST,
+                  port: int=MONITOR_PORT,
+                  console_port: int=CONSOLE_PORT,
+                  console_enabled: bool=True,
+                  locals=OptLocals) -> 'Monitor':
 
     m = Monitor(loop, host=host, port=port, console_port=console_port,
-                console_enabled=console_enabled)
+                console_enabled=console_enabled, locals=locals)
     m.start()
     return m
 
 
 class Monitor:
-    def __init__(self, loop, *, host=MONITOR_HOST, port=MONITOR_PORT,
-                 console_port=CONSOLE_PORT, console_enabled=True):
+    _event_loop_thread_id = None  # type: int
+
+    def __init__(self,
+                 loop: asyncio.AbstractEventLoop, *,
+                 host: str=MONITOR_HOST,
+                 port: int=MONITOR_PORT,
+                 console_port: int=CONSOLE_PORT,
+                 console_enabled: bool=True,
+                 locals: OptLocals=None) -> None:
         self._loop = loop or asyncio.get_event_loop()
         self._host = host
         self._port = port
         self._console_port = console_port
         self._console_enabled = console_enabled
+        self._locals = locals
 
         log.info('Starting aiomonitor at %s:%d', host, port)
 
-        # The monitor launches both a separate thread and helper task
-        # that runs inside curio itself to manage cancellation events
-        self._ui_thread = threading.Thread(target=self.server, args=(),
+        self._ui_thread = threading.Thread(target=self._server, args=(),
                                            daemon=True)
         self._closing = threading.Event()
-        # self._ui_thread.start()
         self._closed = False
         self._started = False
-        self._console_future = None
+        self._console_future = None  # type: Optional[Future[Any]]
 
-    def start(self):
+    def __repr__(self) -> str:
+        name = self.__class__.__name__
+        return '<{name}: {host}:{port}>'.format(
+            name=name, host=self._host, port=self._port)
+
+    def start(self) -> None:
         assert not self._closed
         assert not self._started
 
         self._started = True
         h, p = self._host, self._port
+        self._event_loop_thread_id = threading.get_ident()
         self._ui_thread.start()
         if self._console_enabled:
             log.info('Starting console at %s:%d', h, p)
             self._console_future = init_console_server(
-                self._host, self._console_port, self._loop)
+                self._host, self._console_port, self._locals, self._loop)
 
     @property
-    def closed(self):
+    def closed(self) -> bool:
         return self._closed
 
-    def __enter__(self):
+    def __enter__(self) -> 'Monitor':
         if not self._started:
             self.start()
         return self
 
-    def __exit__(self, type, value, traceback):
+    def __exit__(self, type, value, traceback) -> None:
         self.close()
 
-    def close(self):
+    def close(self) -> None:
         if not self._closed:
             self._closing.set()
             self._ui_thread.join()
-            if self._console_future:
+            if self._console_future is not None:
                 self._console_future.result(timeout=15)
             self._closed = True
 
-    def server(self):
+    def _server(self) -> None:
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         try:
@@ -103,40 +123,43 @@ class Monitor:
                     with client:
                         sout = client.makefile('w', encoding='utf-8')
                         sin = client.makefile('r', encoding='utf-8')
-                        self.interactive_loop(sout, sin)
+                        self._interactive_loop(sout, sin)
                 except (socket.timeout, OSError):
                     continue
 
-    def monitor_commans(self, sin, sout, resp):
+    def _monitor_commans(self, sin: IO[str], sout: IO[str], resp: str) -> None:
         if not resp or resp.startswith(('q', 'exit')):
-            self.command_exit(sout)
+            self._command_exit(sout)
             return
 
         elif resp.startswith('p'):
-            self.command_ps(sout)
+            self._command_ps(sout)
 
         elif resp.startswith('cancel'):
             _, taskid_s = resp.split()
-            self.command_cancel(sout, int(taskid_s))
+            self._command_cancel(sout, int(taskid_s))
 
         elif resp.startswith('signal'):
             _, signame = resp.split()
-            self.command_signal(sout, signame)
+            self._command_signal(sout, signame)
+
+        elif resp.startswith('stacktrace'):
+            self._command_stacktrace(sout)
 
         elif resp.startswith('w'):
             _, taskid_s = resp.split()
-            self.command_where(sout, int(taskid_s))
+            self._command_where(sout, int(taskid_s))
 
         elif resp.startswith('h'):
-            self.command_help(sout)
+            self._command_help(sout)
 
         elif resp.startswith('console'):
-            self.command_console(sin, sout)
+            self._command_console(sin, sout)
 
         else:
             sout.write('Unknown command. Type help.\n')
 
-    def interactive_loop(self, sout, sin):
+    def _interactive_loop(self, sout: IO[str], sin: IO[str]) -> None:
         """Main interactive loop of the monitor
         """
         (sout.write('\nAsyncio Monitor: %d tasks running\n' %
@@ -147,28 +170,29 @@ class Monitor:
             sout.flush()
             try:
                 resp = sin.readline()
-                self.monitor_commans(sin, sout, resp)
+                self._monitor_commans(sin, sout, resp)
             except Exception as e:
                 sout.write('Bad command. %s\n' % e)
                 sout.flush()
 
-    def command_help(self, sout):
+    def _command_help(self, sout: IO[str]):
         sout.write(
-         '''Commands:
+         """Commands:
              ps               : Show task table
              where taskid     : Show stack frames for a task
              cancel taskid    : Cancel an indicated task
              signal signame   : Send a Unix signal
+             stacktrace       : Print a stack trace from the event loop thread
              console          : Switch to async Python REPL
              quit             : Leave the monitor
-            ''')
+            """)
         sout.write('\n')
 
-    def command_ps(self, sout):
+    def _command_ps(self, sout: IO[str]) -> None:
         headers = ('Task ID', 'State', 'Task')
         table_data = [headers]
         for task in sorted(asyncio.Task.all_tasks(loop=self._loop), key=id):
-            taskid = id(task)
+            taskid = str(id(task))
             if task:
                 t = '\n'.join(wrap(str(task), 80))
                 table_data.append((taskid, task._state, t))
@@ -177,7 +201,7 @@ class Monitor:
         sout.write('\n')
         sout.flush()
 
-    def command_where(self, sout, taskid):
+    def _command_where(self, sout: IO[str], taskid: int) -> None:
         task = task_by_id(taskid, self._loop)
         if task:
             sout.write(_format_stack(task))
@@ -185,13 +209,17 @@ class Monitor:
         else:
             sout.write('No task %d\n' % taskid)
 
-    def command_signal(self, sout, signame):
+    def _command_signal(self, sout: IO[str], signame: str) -> None:
         if hasattr(signal, signame):
             os.kill(os.getpid(), getattr(signal, signame))
         else:
             sout.write('Unknown signal %s\n' % signame)
 
-    def command_cancel(self, sout, taskid):
+    def _command_stacktrace(self, sout: IO[str]) -> None:
+        frame = sys._current_frames()[self._event_loop_thread_id]
+        traceback.print_stack(frame, file=sout)
+
+    def _command_cancel(self, sout: IO[str], taskid) -> None:
         task = task_by_id(taskid, self._loop)
         if task:
             fut = asyncio.run_coroutine_threadsafe(
@@ -201,14 +229,15 @@ class Monitor:
         else:
             sout.write('No task %d\n' % taskid)
 
-    def command_exit(self, sout):
+    def _command_exit(self, sout: IO[str]) -> None:
         sout.write('Leaving monitor. Hit Ctrl-C to exit\n')
         sout.flush()
 
-    def command_console(self, sin, sout):
+    def _command_console(self, sin: IO[str], sout: IO[str]) -> None:
         if not self._console_enabled:
             sout.write('Python console disabled for this sessiong\n')
             sout.flush()
 
-        self._console_future.result()
+        if self._console_future is not None:
+            self._console_future.result()
         console_proxy(sin, sout, self._host, self._console_port)
